@@ -102,6 +102,13 @@ class PointField(NumericField, Tiler):
             else:
                 slices.append((tile, tile + shift))
         return Query.any(*itertools.starmap(self.range, slices))
+    def filter(self, lng, lat, distance, lngfield, latfield, limit=Tiler.base):
+        """Return lucene LatLongDistanceFilter based on :meth:`within` query.
+        Alternative distance comparator which requires the spatial contrib module.
+        Distances estimates may be more accurate, and performance may vary.
+        """
+        filter = self.within(lng, lat, distance, limit).filter()
+        return DistanceFilter(filter, lng, lat, distance, lngfield, latfield)
 
 class PolygonField(PointField):
     """PointField which implicitly supports polygons (technically linear rings of points).
@@ -115,3 +122,27 @@ class PolygonField(PointField):
             xs, ys = zip(*(self.project(lat, lng) for lng, lat in points))
             tiles.update(self.walk(min(xs), min(ys), max(xs), max(ys), self.precision))
         return NumericField.items(self, *(int(tile, self.base) for tile in tiles))
+
+class DistanceComparator(Tiler):
+    "Distance comparator computed from cached lat/lngs."
+    def __init__(self, lng, lat, lngs, lats):
+        Tiler.__init__(self)
+        self.x, self.y = self.project(lat, lng)
+        self.lngs, self.lats = lngs, lats
+    def __getitem__(self, id):
+        x, y = self.project(self.lats[id], self.lngs[id])
+        return ((x - self.x)**2 + (y - self.y)**2) ** 0.5
+
+class DistanceFilter(getattr(lucene, 'LatLongDistanceFilter', object)):
+    "Inherited lucene LatLongDistanceFilter which supports the comparator interface."
+    meters = 1609.344
+    def __init__(self, filter, lng, lat, distance, lngfield, latfield):
+        lucene.LatLongDistanceFilter.__init__(self, filter, lat, lng, distance / self.meters, latfield, lngfield)
+    def __getitem__(self, id):
+        distance = self.getDistance(id)
+        if distance is None:
+            raise KeyError(id)
+        return distance.doubleValue() * self.meters
+    def sorter(self, reverse=False):
+        "Return lucene SortField based on the filter's cached distances."
+        return lucene.SortField('distance', lucene.DistanceFieldComparatorSource(self), reverse)
