@@ -21,7 +21,7 @@ Single documents may be added, deleted, or replaced by a unique indexed field.
 Multiples documents may also be added or deleted by query at once.
 By default changes are not visible until the update resource is called to commit a new index version.
 If a near real-time Indexer is used (an experimental feature in Lucene), then changes are instantly searchable.
-In such cases a commit still hasn't occurred;  the index based :meth:`validation headers <validate>` shouldn't be used for caching.
+In such cases a commit still hasn't occurred;  the index based :meth:`last-modified header <validate>` shouldn't be used for caching.
 
  * :meth:`/ <WebIndexer.index>`
  * :meth:`/search <WebIndexer.search>`
@@ -98,9 +98,14 @@ def json_(indent=None, content_type='application/json', process_body=None):
     request.handler = json_handler
 
 @tool('on_start_resource')
-def allow(methods=('GET', 'HEAD')):
+def allow(methods=('GET',), paths=()):
     "Only allow specified methods."
     request = cherrypy.serving.request
+    if paths and hasattr(request.handler, 'args'):
+        with HTTPError(httplib.NOT_FOUND, IndexError):
+            methods = paths[len(request.handler.args)]
+    if 'GET' in methods and 'HEAD' not in methods:
+        methods += 'HEAD',
     if request.method not in methods and not isinstance(request.handler, cherrypy.HTTPError):
         cherrypy.response.headers['allow'] = ', '.join(methods)
         message = "The path {0!r} does not allow {1}.".format(request.path_info, request.method)
@@ -256,8 +261,8 @@ class WebSearcher(object):
         readers = reader.sequentialSubReaders if lucene.MultiReader.instance_(reader) else [reader]
         return dict((unicode(reader.directory()), reader.numDocs()) for reader in readers)
     @cherrypy.expose
-    @cherrypy.tools.params(fields=multi, **{'fields.multi': multi, 'fields.indexed': multi})
-    def docs(self, *path, **options):
+    @cherrypy.tools.params(**dict.fromkeys(['fields', 'fields.multi', 'fields.indexed', 'fields.vector', 'fields.vector.counts'], multi))
+    def docs(self, name=None, value='', **options):
         """Return ids or documents.
         
         **GET** /docs
@@ -267,22 +272,27 @@ class WebSearcher(object):
         
         **GET** /docs/[*int*\|\ *chars*/*chars*]?
             Return document mapping from id or unique name and value.
-            Optionally select stored, multi-valued, and cached indexed fields.
             
             &fields=\ *chars*,... &fields.multi=\ *chars*,... &fields.indexed=\ *chars*\ [:*chars*],...
+                optionally select stored, multi-valued, and cached indexed fields
             
-            :return: {*string*: *string*\|\ *number*\|\ *array*,... }
+            &fields.vector=\ *chars*,... &fields.vector.counts=\ *chars*,...
+                optionally select term vectors with term counts
+            
+            :return: {*string*: null|\ *string*\|\ *number*\|\ *array*\|\ *object*,... }
         """
         searcher = self.searcher
-        if not path:
+        if not name:
             return list(searcher)
         with HTTPError(httplib.NOT_FOUND, ValueError):
-            id, = map(int, path) if len(path) == 1 else searcher.docs(*path)
+            id, = searcher.docs(name, value) if value else [int(name)]
         fields, multi, indexed = params.fields(searcher, **options)
         with HTTPError(httplib.NOT_FOUND, lucene.JavaError):
             doc = searcher[id] if fields is None else searcher.get(id, *itertools.chain(fields, multi))
         result = doc.dict(*multi, **(fields or {}))
         result.update((name, indexed[name][id]) for name in indexed)
+        result.update((field, list(searcher.termvector(id, field))) for field in options.get('fields.vector', ()))
+        result.update((field, dict(searcher.termvector(id, field, counts=True))) for field in options.get('fields.vector.counts', ()))
         return result
     @cherrypy.expose
     @cherrypy.tools.params(count=int, start=int, fields=multi, sort=multi, facets=multi, hl=multi, mlt=int, spellcheck=int, timeout=float,
@@ -522,7 +532,7 @@ class WebIndexer(WebSearcher):
             cherrypy.response.status = httplib.ACCEPTED
     @cherrypy.expose
     @cherrypy.tools.json(process_body=lambda body: {'directories': list(body)})
-    @cherrypy.tools.allow(methods=['GET', 'HEAD', 'POST'])
+    @cherrypy.tools.allow(methods=['GET', 'POST'])
     def index(self, directories=()):
         """Add indexes.  See :meth:`WebSearcher.index` for GET method.
         
@@ -538,8 +548,8 @@ class WebIndexer(WebSearcher):
         return {unicode(self.indexer.directory): len(self.indexer)}
     @cherrypy.expose
     @cherrypy.tools.json(process_body=lambda body: dict.fromkeys(body, True))
-    @cherrypy.tools.allow(methods=['POST', 'PUT', 'DELETE'])
-    def update(self, *path, **options):
+    @cherrypy.tools.allow(paths=[('POST',), ('PUT', 'DELETE')])
+    def update(self, id='', **options):
         """Commit index changes and refresh index version.
         
         **POST** /update
@@ -549,25 +559,24 @@ class WebIndexer(WebSearcher):
             
             :return: *int*
         
-        **PUT, DELETE** /update/snapshot
-            Snapshot current index commit and return array of referenced filenames, or release previous snapshot.
+        **PUT, DELETE** /update/*chars*
+            Create unique snapshot of current index commit and return array of referenced filenames.
+            Release previous snapshot by id.
             
             :return: [*string*,... ]
         """
-        allow(('PUT', 'DELETE') if path else ('GET', 'POST')) # allow direct method call
-        if not path:
+        if not id:
             self.indexer.commit(**options)
             self.updated = time.time()
             return len(self.indexer)
-        path = path[:hasattr(lucene, 'IndexWriterConfig')]
         with HTTPError(httplib.CONFLICT, lucene.JavaError):
             if cherrypy.request.method == 'PUT':
                 cherrypy.response.status = httplib.CREATED
-                return list(self.indexer.policy.snapshot(*path).fileNames)
-            self.indexer.policy.release(*path)
+                return list(self.indexer.policy.snapshot(id).fileNames)
+            self.indexer.policy.release(id)
     @cherrypy.expose
-    @cherrypy.tools.allow(methods=['GET', 'HEAD', 'POST', 'PUT', 'DELETE'])
-    def docs(self, *path, **options):
+    @cherrypy.tools.allow(paths=[('GET', 'POST'), ('GET',), ('GET', 'PUT', 'DELETE')])
+    def docs(self, name=None, value='', **options):
         """Add or return documents.  See :meth:`WebSearcher.docs` for GET method.
         
         **POST** /docs
@@ -580,15 +589,12 @@ class WebIndexer(WebSearcher):
             
             {*string*: *string*\|\ *number*\|\ *array*,... }
         """
-        with HTTPError(httplib.NOT_FOUND, IndexError):
-            allow([('GET', 'HEAD', 'POST'), ('GET', 'HEAD'), ('GET', 'HEAD', 'PUT', 'DELETE')][len(path)])
         request = cherrypy.serving.request
         if request.method in ('GET', 'HEAD'):
-            return WebSearcher.docs(self, *path, **options)
+            return WebSearcher.docs(self, name, value, **options)
         if request.method == 'DELETE':
-            self.indexer.delete(*path)
+            self.indexer.delete(name, value)
         elif request.method == 'PUT':
-            name, value = path
             doc = getattr(request, 'json', {})
             with HTTPError(httplib.CONFLICT, KeyError, AssertionError):
                 assert self.indexer.fields[name].index.indexed, 'unique field must be indexed'
@@ -601,7 +607,7 @@ class WebIndexer(WebSearcher):
         self.refresh()
     docs._cp_config.update(WebSearcher.docs._cp_config)
     @cherrypy.expose
-    @cherrypy.tools.allow(methods=['GET', 'HEAD', 'DELETE'])
+    @cherrypy.tools.allow(methods=['GET', 'DELETE'])
     def search(self, q=None, **options):
         """Run or delete a query.  See :meth:`WebSearcher.search` for GET method.
         
@@ -618,7 +624,7 @@ class WebIndexer(WebSearcher):
     search._cp_config.update(WebSearcher.search._cp_config)
     @cherrypy.expose
     @cherrypy.tools.json(process_body=dict)
-    @cherrypy.tools.allow(methods=['GET', 'HEAD', 'PUT'])
+    @cherrypy.tools.allow(paths=[('GET',), ('GET', 'PUT')])
     @cherrypy.tools.validate(on=False)
     def fields(self, name='', **settings):
         """Return or store a field's parameters.
@@ -636,7 +642,6 @@ class WebIndexer(WebSearcher):
             :return: {"store": *string*, "index": *string*, "termvector": *string*}
         """
         if not name:
-            allow()
             return sorted(self.indexer.fields)
         if cherrypy.request.method == 'PUT':
             if name not in self.indexer.fields:
@@ -694,14 +699,18 @@ parser.add_option('-r', '--read-only', action='store_true', help='expose only re
 parser.add_option('-c', '--config', help='optional configuration file or json object of global params')
 parser.add_option('-p', '--pidfile', metavar='FILE', help='store the process id in the given file')
 parser.add_option('-d', '--daemonize', action='store_true', help='run the server as a daemon')
-parser.add_option('--autoreload', type=int, metavar='SECONDS', help='automatically reload modules; replacement for engine.autoreload')
-parser.add_option('--autoupdate', type=int, metavar='SECONDS', help='automatically update index version')
+parser.add_option('--autoreload', type=float, metavar='SECONDS', help='automatically reload modules; replacement for engine.autoreload')
+parser.add_option('--autoupdate', type=float, metavar='SECONDS', help='automatically update index version and commit any changes')
+parser.add_option('--real-time', action='store_true', help='search in real-time without committing')
 
 if __name__ == '__main__':
     options, args = parser.parse_args()
-    read_only = options.__dict__.pop('read_only')
+    read_only = options.__dict__.pop('read_only') or len(args) > 1
+    kwargs = {'nrt': True} if options.__dict__.pop('real_time') else {}
+    if read_only and (kwargs or not args):
+        parser.error('incompatible read/write options')
     if options.config and not os.path.exists(options.config):
         options.config = {'global': json.loads(options.config)}
-    cls = WebSearcher if (read_only or len(args) > 1) else WebIndexer
-    root = cls.new(*map(os.path.abspath, args))
+    cls = WebSearcher if read_only else WebIndexer
+    root = cls.new(*map(os.path.abspath, args), **kwargs)
     start(root, callback=init, **options.__dict__)
