@@ -75,16 +75,14 @@ class TestCase(BaseTest):
         assert indexer.current and indexer.optimized
         assert 0 in indexer and 1 not in indexer
         doc = indexer[0]
-        assert len(doc) == 3
-        assert 'name' in doc and 'missing' not in doc
-        assert sorted(doc) == ['name', 'tag', 'tag']
-        assert sorted(doc.items()) == [('name', 'sample'), ('tag', 'python'), ('tag', 'search')]
+        assert doc == {'tag': ['python', 'search'], 'name': ['sample']}
+        assert doc['name'] == 'sample' and doc['tag'] == 'python'
         assert doc.dict('tag') == {'name': 'sample', 'tag': ['python', 'search']}
         assert doc.dict(name=None, missing=True) == {'name': 'sample', 'missing': True}
-        doc['name'] == 'sample' and doc['tag'] in ('python', 'search')
         self.assertRaises(KeyError, doc.__getitem__, 'key')
         assert doc.getlist('name') == ['sample'] and doc.getlist('key') == []
         assert indexer.get(0, 'name').dict() == {'name': 'sample'}
+        assert not list(indexer.termvector(0, 'tag'))
         assert indexer.count('text', 'hello') == indexer.count('text:hello') == 1
         assert sorted(indexer.names()) == ['name', 'tag', 'text']
         assert sorted(indexer.names('indexed')) == ['tag', 'text']
@@ -121,8 +119,7 @@ class TestCase(BaseTest):
         assert str(query) == '+text:* +text:hello~0.5 text:hello~0.1'
         query = engine.Query.span('text', 'world')
         assert str(query.mask('name')) == 'mask(text:world) as name'
-        if hasattr(lucene, 'SpanPositionCheckQuery'):
-            assert str(query.payload()) == 'spanPayCheck(text:world, payloadRef: )'
+        assert str(query.payload()) == 'spanPayCheck(text:world, payloadRef: )'
         query = engine.Query.disjunct(0.1, query, name='sample')
         assert str(query) == '(text:world | name:sample)~0.1'
         query = engine.Query.near('text', 'hello', ('tag', 'python'), slop=-1, inOrder=False)
@@ -131,10 +128,9 @@ class TestCase(BaseTest):
         (doc, items), = indexer.spans(query, payloads=True)
         (start, stop, payloads), = items
         assert doc == 0 and start == 0 and stop == 2 and payloads == ['<ALPHANUM>', '<ALPHANUM>']
-        if hasattr(lucene, 'SpanPositionCheckQuery'):
-            (doc, count), = indexer.spans(query.payload('<ALPHANUM>', '<ALPHANUM>'))
-            assert doc == 0 and count == 1
-            assert not indexer.search(query.payload('<>'))
+        (doc, count), = indexer.spans(query.payload('<ALPHANUM>', '<ALPHANUM>'))
+        assert doc == 0 and count == 1
+        assert not indexer.search(query.payload('<>'))
         query = engine.Query.near('text', 'hello', 'world', collectPayloads=False)
         (doc, items), = indexer.spans(query, payloads=True)
         assert doc == 0 and items == []
@@ -159,9 +155,9 @@ class TestCase(BaseTest):
         indexer += temp.directory
         indexer += self.tempdir
         assert len(indexer) == 3
-        indexer.add(text=lucene.WhitespaceTokenizer(lucene.StringReader('?')))
+        indexer.add(text=lucene.WhitespaceTokenizer(lucene.StringReader('?')), name=lucene.JArray_byte('{}'))
         indexer.commit()
-        assert list(indexer.terms('text')) == ['?']
+        assert indexer[next(indexer.docs('text', '?'))] == {'name': ['{}']}
         reader = engine.indexers.IndexReader(indexer.indexReader)
         assert reader[0].dict() == {} and reader.count('text', '?') == 1
         assert len(reader.comparator('text')) == 4
@@ -265,7 +261,8 @@ class TestCase(BaseTest):
         hits = indexer.search('text:right')
         for name in ('amendment', 'article'):
             indexer.filters[name] = engine.Query.prefix(name, '').filter()
-        query = engine.Query.term('text', 'right')
+        query = engine.Query.term('text', 'right', boost=2.0)
+        assert query.boost == 2.0
         assert indexer.facets(str(query), 'amendment', 'article') == {'amendment': 12, 'article': 1}
         self.assertRaises(TypeError, indexer.overlap, query.filter(), query.filter(cache=False))
         hits = indexer.search('text:people', filter=query.filter())
@@ -276,8 +273,6 @@ class TestCase(BaseTest):
         amendments = ['18', '19']
         assert sorted(hit['amendment'] for hit in hits) == amendments
         query = engine.Query.range('date', '1919', '1921')
-        query.boost = 2.0
-        assert query.boost == 2.0
         hits = indexer.search(query)
         assert sorted(hit['amendment'] for hit in hits) == amendments
         hits = indexer.search(query | engine.Query.term('text', 'vote'))
@@ -301,15 +296,13 @@ class TestCase(BaseTest):
         assert len(query) == len(list(query)) == 2
         span = engine.Query.span('text', 'persons')
         count = indexer.count(span)
-        if hasattr(lucene, 'SpanMultiTermQueryWrapper'):
-            assert indexer.count(engine.Query.span(engine.Query.prefix('text', 'person'))) > count
+        assert indexer.count(engine.Query.span(engine.Query.prefix('text', 'person'))) > count
         near = engine.Query.near('text', 'persons', 'papers', slop=1, inOrder=False)
         assert indexer.count(span - near) == count
         near = span.near(engine.Query.span('text', 'papers') | engine.Query.span('text', 'things'), slop=1)
         assert indexer.count(span - near) == count - 1
         assert 0 < indexer.count(span[:100]) < count
-        if hasattr(lucene, 'SpanPositionRangeQuery'):
-            assert 0 < indexer.count(span[50:100]) == indexer.count(span[:100] - span[:50]) < indexer.count(span[:100])
+        assert 0 < indexer.count(span[50:100]) == indexer.count(span[:100] - span[:50]) < indexer.count(span[:100])
         spans = dict(indexer.spans(span))
         assert len(spans) == count and spans == dict(indexer.docs('text', 'persons', counts=True))
         near = engine.Query.near('text', 'persons', 'papers', slop=2)
@@ -463,6 +456,7 @@ class TestCase(BaseTest):
         counts = dict((hits.value, len(hits)) for hits in groups)
         assert 1 == counts[0] < counts[2] < counts[1]
         assert len(field.within(x, y, 10**8)) == 1
+        self.assertRaises(NameError, list, field.radiate(y, x, 1, 0))
         if hasattr(lucene, 'LatLongDistanceFilter'):
             f = field.filter(x, y, 10**4, 'longitude', 'latitude')
             ids = indexer.search(query, sort=distances.__getitem__).ids
@@ -603,13 +597,10 @@ class TestCase(BaseTest):
         fragment, = indexer.highlighter(query, 'text', tag='em').fragments(id, count=3)
         assert len(fragment) < len(text) and fragment.index('<em>persons') < fragment.index('papers</em>')
         fragment, = indexer.highlighter(query, 'text').fragments(id)
-        if hasattr(lucene, 'FastVectorHighlighter'):
-            assert fragment.count('<b>') == fragment.count('</b>') == 1
-            highlighter = indexer.highlighter(query, 'text', fragListBuilder=lucene.SingleFragListBuilder())
-            text, = highlighter.fragments(id)
-            assert fragment in text and len(text) > len(fragment)
-        else:
-            assert fragment.count('<B>') == fragment.count('</B>') == 3
+        assert fragment.count('<b>') == fragment.count('</b>') == 1
+        highlighter = indexer.highlighter(query, 'text', fragListBuilder=lucene.SingleFragListBuilder())
+        text, = highlighter.fragments(id)
+        assert fragment in text and len(text) > len(fragment)
     
     def testNearRealTime(self):
         "Near real-time index updates."
