@@ -68,7 +68,7 @@ class TestCase(BaseTest):
         "Remote reading and writing."
         self.servers += (
             self.start(self.ports[0], self.tempdir, '--autoreload=1', **{'tools.validate.expires': 0, 'tools.validate.max_age': 0}),
-            self.start(self.ports[1], self.tempdir, self.tempdir, '--autoupdate=2'), # concurrent searchers
+            self.start(self.ports[1], self.tempdir, self.tempdir, '--autoupdate=2.0'), # concurrent searchers
         )
         resource = client.Resource('localhost', self.ports[0])
         assert resource.get('/favicon.ico')
@@ -253,6 +253,7 @@ class TestCase(BaseTest):
         response = resource.call('PUT', '/update/snapshot')
         assert response.status == httplib.CREATED
         assert all(name.startswith('_') or name.startswith('segments_') for name in response())
+        assert resource.put('/update/backup') == response()
         with assertRaises(httplib.HTTPException, httplib.CONFLICT):
             resource.put('/update/snapshot')
         assert not resource.delete('/update/snapshot')
@@ -263,10 +264,14 @@ class TestCase(BaseTest):
             resource.get('/')
         self.stop(self.servers.pop(0))
         pidfile = os.path.join(self.tempdir, 'pid')
-        server = self.start(self.ports[0], '-dp', pidfile)
-        os.kill(int(open(pidfile).read()), signal.SIGKILL)
+        self.start(self.ports[0], '-dp', pidfile)
+        time.sleep(1)
+        os.kill(int(open(pidfile).read()), signal.SIGTERM)
         filepath = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'lupyne/server.py')
-        assert subprocess.call((sys.executable, filepath, '-x'), stderr=subprocess.PIPE)
+        assert subprocess.call((sys.executable, filepath, '-c', filepath), stderr=subprocess.PIPE)
+        assert server.mount(None, '/path', config={'': {}})
+        server.init(vmargs=None)
+        self.assertRaises(AttributeError, server.start, config=True)
     
     def testBasic(self):
         "Remote text indexing and searching."
@@ -283,6 +288,10 @@ class TestCase(BaseTest):
         assert resource.get('/').values() == [35]
         resource.post('/update', ['optimize', 'spellcheckers'])
         assert resource.get('/docs/0', **{'fields.indexed': 'amendment:int'}) == {'amendment': 0, 'article': 'Preamble'}
+        doc = resource.get('/docs/0', **{'fields.vector': 'text,missing'})
+        assert doc['missing'] == [] and doc['text'].index('states') < doc['text'].index('united')
+        doc = resource.get('/docs/0', **{'fields.vector.counts': 'text'})
+        assert sorted(term for term, count in doc['text'].items() if count > 1) == ['establish', 'states', 'united']
         assert resource.get('/terms') == ['amendment', 'article', 'date', 'text']
         articles = resource.get('/terms/article')
         articles.remove('Preamble')
@@ -435,17 +444,15 @@ class TestCase(BaseTest):
         assert all(group['value'].startswith('Los Angeles') for group in result['groups'])
         assert sum(map(operator.itemgetter('count'), result['groups'])) == sum(facets.values()) == result['count']
     
-    def testNearRealTime(self):
-        "Near Real Time updating and searching."
-        with contextlib.closing(engine.IndexWriter(self.tempdir)) as writer:
-            writer.add()
+    def testRealTime(self):
+        "Real Time updating and searching."
+        for args in [('-r',), ('--real-time', 'index0', 'index1'), ('-r', '--real-time', 'index')]:
+            assert subprocess.call((sys.executable, '-m', 'lupyne.server') + args, stderr=subprocess.PIPE)
+        with contextlib.closing(server.WebIndexer(self.tempdir)) as root:
+            root.indexer.add()
+            assert root.update() == 1
         port = self.ports[0]
-        cherrypy.engine.subscribe('start_thread', server.attach_thread)
-        cherrypy.config.update(self.config)
-        cherrypy.config.update({'engine.autoreload.on': False, 'server.socket_port': port, 'tools.validate.expires': 0, 'tools.validate.last_modified': False})
-        root = server.WebIndexer(nrt=True)
-        cherrypy.tree.mount(root, config={'global': {}})
-        cherrypy.engine.start()
+        self.servers.append(self.start(self.ports[0], '--real-time', **{'tools.validate.expires': 0, 'tools.validate.last_modified': False}))
         resource = client.Resource('localhost', port)
         response = resource.call('GET', '/docs')
         version, modified, expires = map(response.getheader, ('etag', 'last-modified', 'expires'))
@@ -461,8 +468,6 @@ class TestCase(BaseTest):
         resource.post('/update')
         response = resource.call('GET', '/docs')
         assert response and version != response.getheader('etag')
-        cherrypy.engine.exit()
-        assert root.update() == 1
 
 if __name__ == '__main__':
     lucene.initVM()
